@@ -58,6 +58,11 @@ from wechaty_puppet import (  # type: ignore
 from wechaty_puppet.schemas.puppet import PUPPET_EVENT_DICT, PuppetOptions  # type: ignore
 from wechaty_puppet.state_switch import StateSwitch     # type: ignore
 from wechaty_puppet.watch_dog import WatchdogFood, Watchdog     # type: ignore
+
+from .plugin import (
+    WechatyPlugin, WechatyPluginManager
+)
+
 from .user import (
     Contact,
     Friendship,
@@ -65,12 +70,17 @@ from .user import (
     Tag,
     Room,
     Image,
-    RoomInvitation
+    RoomInvitation,
+    MiniProgram,
+    Favorite,
+    ContactSelf
 )
+
 from .utils import (
     qr_terminal
 )
 
+
 log = get_logger('Wechaty')
 
 DEFAULT_TIMEOUT = 300
@@ -113,13 +123,25 @@ class Wechaty(AsyncIOEventEmitter):
         if options.puppet_options is None:
             options.puppet_options = PuppetOptions()
 
+        # pylint: disable=C0103
         self.Tag = Tag
+        # pylint: disable=C0103
         self.Contact = Contact
+        # pylint: disable=C0103
+        self.ContactSelf = ContactSelf
+        # pylint: disable=C0103
         self.Friendship = Friendship
+        # pylint: disable=C0103
         self.Message = Message
+        # pylint: disable=C0103
         self.Room = Room
+        # pylint: disable=C0103
         self.Image = Image
+        # pylint: disable=C0103
         self.RoomInvitation = RoomInvitation
+        self.Favorite = Favorite
+        self.MiniProgram = MiniProgram
+        # TODO -> url-link, miniprogram
 
         self.started: bool = False
 
@@ -132,6 +154,8 @@ class Wechaty(AsyncIOEventEmitter):
 
         self._watchdog = Watchdog(DEFAULT_TIMEOUT)
 
+        self._plugin_manager: WechatyPluginManager = WechatyPluginManager(self)
+
     @staticmethod
     def _load_puppet(options: WechatyOptions) -> Puppet:
         """
@@ -189,6 +213,16 @@ class Wechaty(AsyncIOEventEmitter):
         # return cast(Wechaty, cls._global_instance)
         # return cls._global_instance
 
+    def use(self, plugin: Union[WechatyPlugin, List[WechatyPlugin]]) -> Wechaty:
+        """register the plugin"""
+        if isinstance(plugin, WechatyPlugin):
+            plugins = [plugin]
+        else:
+            plugins = plugin
+        for item in plugins:
+            self._plugin_manager.add_plugin(item)
+        return self
+
     @property
     def name(self) -> str:
         """name"""
@@ -368,9 +402,12 @@ class Wechaty(AsyncIOEventEmitter):
                 puppet.on('dong', dong_listener)
             elif event_name == 'error':
                 async def error_listener(payload: EventErrorPayload):
-                    log.info('receive <error> event <%s>', payload)
-                    self.emit('error', payload)
-                    await self.on_error(payload)
+                    if isinstance(payload, EventErrorPayload):
+                        log.info('receive <error> event <%s>', payload)
+                        self.emit('error', payload)
+                        await self.on_error(payload)
+                    else:
+                        log.error('internal error <%s>', payload)
 
                 puppet.on('error', error_listener)
 
@@ -403,6 +440,9 @@ class Wechaty(AsyncIOEventEmitter):
                     self.emit('login', Contact)
                     await self.on_login(contact)
 
+                    # init the plugins
+                    await self._plugin_manager.init_plugins()
+
                 puppet.on('login', login_listener)
 
             elif event_name == 'logout':
@@ -413,7 +453,6 @@ class Wechaty(AsyncIOEventEmitter):
                     await contact.ready()
                     self.emit('logout', Contact)
                     await self.on_logout(contact)
-
                 puppet.on('logout', logout_listener)
 
             elif event_name == 'message':
@@ -429,6 +468,8 @@ class Wechaty(AsyncIOEventEmitter):
                     if room is not None:
                         room.emit('message', room)
 
+                    await self._plugin_manager.emit_events('message', msg)
+
                 puppet.on('message', message_listener)
 
             elif event_name == 'ready':
@@ -531,8 +572,8 @@ class Wechaty(AsyncIOEventEmitter):
                         else payload.qrcode
                     if payload.status == ScanStatus.Waiting:
                         qr_terminal(qr_code)
-                    self.emit('scan', payload.status, qr_code,  payload.data)
-                    await self.on_scan(payload.status, qr_code,  payload.data)
+                    self.emit('scan', payload.status, qr_code, payload.data)
+                    await self.on_scan(payload.status, qr_code, payload.data)
 
                 puppet.on('scan', scan_listener)
 
@@ -558,11 +599,17 @@ class Wechaty(AsyncIOEventEmitter):
         self.Room.set_puppet(self.puppet)
         self.RoomInvitation.set_puppet(self.puppet)
         self.Contact.set_puppet(self.puppet)
+        self.Friendship.set_puppet(self.puppet)
+        self.Image.set_puppet(self.puppet)
+        self.Tag.set_puppet(self.puppet)
 
         self.Message.set_wechaty(self)
         self.Room.set_wechaty(self)
         self.RoomInvitation.set_wechaty(self)
         self.Contact.set_wechaty(self)
+        self.Friendship.set_wechaty(self)
+        self.Image.set_wechaty(self)
+        self.Tag.set_wechaty(self)
 
     async def stop(self):
         """
@@ -570,3 +617,19 @@ class Wechaty(AsyncIOEventEmitter):
         """
         log.info('wechaty is stoping ...')
         await self.puppet.stop()
+
+    def user_self(self) -> ContactSelf:
+        """
+        get user self
+        :return:
+        """
+        user_id = self.puppet.self_id()
+        user = self.ContactSelf.load(user_id)
+        return user
+
+    def self(self) -> Contact:
+        """
+        get user self
+        :return: user_self
+        """
+        return self.user_self()
